/**@name SCRIPT: unLHC_CIRCUIT_qpsMacro.ctl

@author: Herve Milcent (AB-CO)

Creation Date: 11/08/2008

Modification History: 

  2015/08/18: QPS-219 - 13kA open switch macro, commands with same ID. Added delay of 3 msec.

  03/03/2015: QPS-203 - FW: 13kA Swicth opening

  24/06/2014: Herve
  - ENS-11589 - Collect all HDS_INTERLOCK and QPS_COMM from each QPS projects and publish the result in QPS.PVSS.HDS_INTERLOCK_xx and QPS.PVSS.QPS_COMM_xx

  01/03/2011: Herve
  - CIR-45 QPS CSCM macro
  
  16/11/2011: Herve
  - CIR-35 PVSS LHCCircuit scripts: system name and component version added in the MessageText log
  
  -27/05/2010: Herve
  LHC_CIRCUITMACRO_AERO_CALIBRATE_ON_TIMEOUT set to 110
  
  - 29/10/2009: Herve
  increase LHC_CIRCUITMACRO_SNAPSHOT_TIMEOUT to 510 (1 minute delay between each snapshot)
  
  - 22/07/2009:
    add Snapshot and Zero_Calibrate_ON macro
    
  - 05/06/2009:
    add SOFT_FE system integrity and debug
    
  15/08/2008: Herve
    - add delay CB
    - add abort CB
    
version 1.1

Purpose: 
LHC_CIRCUIT interface for QPS macro to 
  - close the switch of a set of circuit
  - reset QPS agent
  - execute the snapshot macro
  - execute the Zero calibrate macro 
  
close switch macro:
  - wait for list of circuit sent by LHCOP application on LHC_CIRCUIT.CLOSE_SWITCH.CIRCUIT_LIST
  and send it to the QPS system on QPS_LHC_CIRCUIT.CLOSE_SWITCH.CIRCUIT_LIST with the current value 
  LHC_CIRCUIT.CLOSE_SWITCH.RESET_SYST to QPS_LHC_CIRCUIT.CLOSE_SWITCH.RESET_SYST
reset agent:
  - wait for list of circuit  sent by LHCOP application on LHC_CIRCUIT.RESET.CIRCUIT_LIST 
  and send it to the QPS system on QPS_LHC_CIRCUIT.RESET.CIRCUIT_LIST
Snapshot macro:
  - wait for list of circuit sent by LHCOP application on LHC_CIRCUIT_SNAPSHOT.MACRO.SET and send it to 
  the QPS system on QPS_LHC_CIRCUIT_SNAPSHOT.MACRO.SET
  - wait for the result on QPS_LHC_CIRCUIT_SNAPSHOT.MACRO.RESULT from the involved QPS system 
  - set the result on LHC_CIRCUIT_SNAPSHOT.MACRO.RESULT
  - if LHC_CIRCUIT_SNAPSHOT.MACRO.ABORT is set transmit it to all the QPS system
Zero_Calibrate_ON macro:
  - wait for list of circuit sent by LHCOP application on LHC_CIRCUIT_ZEROCALIBRATEON.MACRO.SET and send it to 
  the QPS system on QPS_LHC_CIRCUIT_ZEROCALIBRATEON.MACRO.SET
  - wait for the result on QPS_LHC_CIRCUIT_ZEROCALIBRATEON.MACRO.RESULT from the involved QPS system 
  - set the result on LHC_CIRCUIT_ZEROCALIBRATEON.MACRO.RESULT
  - if LHC_CIRCUIT_ZEROCALIBRATEON.MACRO.ABORT is set transmit it to all the QPS system

Usage: LHC Op.

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.6 
	. operating system: WXP and Linux.
	. distributed system: no.
*/
// libs
#uses "unTreeDeviceOverview.ctl"
#uses "CtrlTreeDeviceOverview"
#uses "unLHCCircuit_TreeOverview.ctl"
#uses "qpsLHC_CircuitMacro.ctl"
#uses "unGenericObject.ctl"


// constant
const string QPS_SNAPSHOT_MACRO_RESULT_DPE = ":QPS_LHC_CIRCUIT_SNAPSHOT.MACRO.RESULT";
const string QPS_ZERO_CALIBRATE_ON_MACRO_RESULT_DPE = ":QPS_LHC_CIRCUIT_ZEROCALIBRATEON.MACRO.RESULT";
const string QPS_RESET_MACRO_RESULT_DPE = ":QPS_LHC_CIRCUIT_RESET.MACRO.RESULT";
const string QPS_CSCM_MACRO_RESULT_DPE = ":QPS_LHC_CIRCUIT_CSCM.MACRO.RESULT";
const string QPS_SELECT_BOARD_A_MACRO_RESULT_DPE = ":QPS_LHC_CIRCUIT_SELECT_BOARD_A.MACRO.RESULT";
const string QPS_SELECT_BOARD_B_MACRO_RESULT_DPE = ":QPS_LHC_CIRCUIT_SELECT_BOARD_B.MACRO.RESULT";
const string QPS_PREPARE_TEST_MODE_MACRO_RESULT_DPE = ":QPS_LHC_CIRCUIT_PREPARE_TEST_MODE.MACRO.RESULT";
const string QPS_CANCEL_TEST_MODE_MACRO_RESULT_DPE = ":QPS_LHC_CIRCUIT_CANCEL_TEST_MODE.MACRO.RESULT";
const int LHC_CIRCUITMACRO_SNAPSHOT_TIMEOUT = 510; // max time allowed for the Snapshot macro
const int LHC_CIRCUITMACRO_AERO_CALIBRATE_ON_TIMEOUT=110; // max time allowed for the Zero_Calibrate_ON macro
const int LHC_CIRCUITMACRO_RESET_TIMEOUT=80; // max time allowed for the reset macro
const int LHC_CIRCUITMACRO_SELECT_BOARD_TIMEOUT=80; // max time allowed for the reset macro
const int LHC_CIRCUITMACRO_TEST_MODE_TIMEOUT=80; // max time allowed for the reset macro
const int LHC_CIRCUITMACRO_OPEN_SWITCH_DELAY_BEFORE_START = 3; // delay to wait before starting the open switch macro

const int QPS_HDS_INTERLOCK_STATUS = 1;
const int QPS_HDS_INTERLOCK_WARNING_STATUS = 2;
const int QPS_HDS_COMM_STATUS = 3;
const int QPS_HDS_INTERLOCK_WAIT = 1;

// global var
bool g_bResetSyst;
string g_sSystemName;
mapping g_mMacroResult; // result variable
mapping g_mMacroSystemList; // list of system in use per macro
dyn_string g_dsCircuitRunningMacro; // list of circuit with a macro currently running
mapping g_m_bConnected;
mapping g_mHDS_COMM; // keep here the HDS_INTERLOCK status&warningStatus and QPS_COMM status for each system name
int g_iThreadId;
time g_tTimeCounter;

/*
  Description of the execution of a macro:
  
  g_mMacroResult and g_mMacroSystemList have the same keys. The key is ID+macro, ID is the ID given by the 
  trigger of the macro and macro is the running macro. ID+macro will be the ID for the QPS macro.
  g_mMacroResult: key=ID+macro, value=result. At the end of the macro the dynlen of value is the same as 
  the dynlen of the involved system.
  g_mMacroSystemList: key=ID+macro, value=list of system involved in the macro, at startup of the macro the 
  list of system is set for the corresponding key. As soon as a QPS system ends the macro, the result is set
  and the callback is triggered (see below)
  At the end of the macro this list will be empty. This variable is important because one can have the same 
  macro started on different circuit, so on different QPS system. The result is via a permanent callback and
  one has to distinguish for which ID the result is for. On can also have the same system triggered with 
  different ID and different circuit.
  g_dsCircuitRunningMacro: list of circuit executing a macro. The same circuit can execute only one macro at the
  same time.
  
  at startup the scritp registers to all defined systems
  
  whenever a disconnection is triggered 
  (unLHC_CIRCUIT_qpsMacro_systemCB), each callback checks if the system in disconnection state 
  is in the list of system in use for all the keys of g_mMacroSystemList, if so for the corresponding key 
  it sets in g_mMacroResult "Error system disconnected" and remove itself from g_mMacroSystemList
  
  whenever a connection callback is triggered (unLHC_CIRCUIT_qpsMacro_systemCB), each callback 
  does a dpConnect to the snapshot macro result (unLHC_CIRCUIT_qpsMacro_ResultCB) 
  and zero calibrate QPS macro result (unLHC_CIRCUIT_qpsMacro_ResultCB) if both dp exist.
  When the result of the QPS dp is set, the callbacks unLHC_CIRCUIT_qpsMacro_Snapshot_ResultCB 
  or unLHC_CIRCUIT_qpsMacro_Zero_Calibrate_ON_ResultCB checks if the system is in the list of the system 
  in use: g_mMacroSystemList for the key of the result (key = ID+macro): it sets the result into g_mMacroResult 
  and remove itself from g_mMacroSystemList for the corresponding key.
   
  when a macro is started, the given ID+macro with the list of circuit is sent to the involved system
  the list of cicruit executing a macro is kept into g_dsCircuitRunningMacro
  periodically a check is done on g_mMacroResult: the number of message must be the same as the number of
  involved system
*/

//@{
//------------------------------------------------------------------------------------------------------------------------
// main
/**
Purpose:
This is the main of the script.

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.6 
	. operating system: XP and Linux
	. distributed system: no.
*/
main()
{
  string sSystemName=getSystemName();
  dyn_string exceptionInfo;
  dyn_string dsPWS, dsCircuit, dsInterlockType, dsCircuitType, dsBuilding;
  int iDeviceNumber;
  
  // init the tree global var
  unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                    "unLHC_CIRCUIT_qpsMacro starting ...", exceptionInfo);
  if(strpos(sSystemName, ":") < 0)
    sSystemName = sSystemName+":";
  g_sSystemName = sSystemName;

  unLHCCircuit_TreeOverview_getCircuitDeviceProperty(sSystemName, dsPWS, dsCircuit, dsInterlockType, 
                                                     dsCircuitType, dsBuilding, iDeviceNumber);
  g_dsLHC_CIRCUITCircuit = dsCircuit;
  g_dsLHC_CIRCUITCircuitInterlockType = dsInterlockType;
  g_dsLHC_CIRCUITCircuitCircuitType = dsCircuitType;

  unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                    "unLHC_CIRCUIT_qpsMacro init done, ready for operation ...", exceptionInfo);
  // register to all systems
  unLHC_CIRCUIT_qpsMacro_registerToAllSystem();
  g_iThreadId = startThread("unLHC_CIRCUIT_qpsMacro_calculateState_HDS_COMM");
  
  // connect to the internal DP
  dpConnect("unLHC_CIRCUIT_qpsMacro_closeSwitchDelayCB", 0, "LHC_CIRCUIT.CLOSE_SWITCH.CLOSE_DELAY",
                                                  "LHC_CIRCUIT.CLOSE_SWITCH.RESET_SYST_DELAY");
  dpConnect("unLHC_CIRCUIT_qpsMacro_closeSwitchAbortCB", 0, "LHC_CIRCUIT.CLOSE_SWITCH.ABORT");
  dpConnect("unLHC_CIRCUIT_qpsMacro_ResetSwitchCB", "LHC_CIRCUIT.CLOSE_SWITCH.RESET_SYST");
  dpConnect("unLHC_CIRCUIT_qpsMacro_closeSwitchCB", 0, "LHC_CIRCUIT.CLOSE_SWITCH.CIRCUIT_LIST");
  dpConnect("unLHC_CIRCUIT_qpsMacro_resetAgentCB", 0, "LHC_CIRCUIT.RESET.CIRCUIT_LIST");
  dpConnect("unLHC_CIRCUIT_qpsMacro_Snapshot_CB", 0, "LHC_CIRCUIT_SNAPSHOT.MACRO.SET");
  dpConnect("unLHC_CIRCUIT_qpsMacro_Abort_Snapshot_CB", 0, "LHC_CIRCUIT_SNAPSHOT.MACRO.ABORT");
  dpConnect("unLHC_CIRCUIT_qpsMacro_Zero_Calibrate_ON_CB", 0, "LHC_CIRCUIT_ZEROCALIBRATEON.MACRO.SET");
  dpConnect("unLHC_CIRCUIT_qpsMacro_Abort_Zero_Calibrate_ON_CB", 0, "LHC_CIRCUIT_ZEROCALIBRATEON.MACRO.ABORT");
  dpConnect("unLHC_CIRCUIT_qpsMacro_Reset_CB", 0, "LHC_CIRCUIT_RESET.MACRO.SET");
  dpConnect("unLHC_CIRCUIT_qpsMacro_CSCM_CB", 0, "LHC_CIRCUIT_CSCM.MACRO.SET");
  dpConnect("unLHC_CIRCUIT_qpsMacro_Abort_CSCM_CB", 0, "LHC_CIRCUIT_CSCM.MACRO.ABORT");
  
  dpConnect("unLHC_CIRCUIT_qpsMacro_SelectBoardA_CB", 0, "LHC_CIRCUIT_SELECT_BOARD_A.MACRO.SET");
  dpConnect("unLHC_CIRCUIT_qpsMacro_Abort_SelectBoardA_CB", 0, "LHC_CIRCUIT_SELECT_BOARD_A.MACRO.ABORT");
  dpConnect("unLHC_CIRCUIT_qpsMacro_SelectBoardB_CB", 0, "LHC_CIRCUIT_SELECT_BOARD_B.MACRO.SET");
  dpConnect("unLHC_CIRCUIT_qpsMacro_Abort_SelectBoardB_CB", 0, "LHC_CIRCUIT_SELECT_BOARD_B.MACRO.ABORT");
  dpConnect("unLHC_CIRCUIT_qpsMacro_PrepareTestMode_CB", 0, "LHC_CIRCUIT_PREPARE_TEST_MODE.MACRO.SET");
  dpConnect("unLHC_CIRCUIT_qpsMacro_Abort_PrepareTestMode_CB", 0, "LHC_CIRCUIT_PREPARE_TEST_MODE.MACRO.ABORT");
  dpConnect("unLHC_CIRCUIT_qpsMacro_CancelTestMode_CB", 0, "LHC_CIRCUIT_CANCEL_TEST_MODE.MACRO.SET");
  dpConnect("unLHC_CIRCUIT_qpsMacro_Abort_CancelTestMode_CB", 0, "LHC_CIRCUIT_CANCEL_TEST_MODE.MACRO.ABORT");
  dpConnect("unLHC_CIRCUIT_qpsMacro_OpenSwitch_CB", 0, "LHC_CIRCUIT_OPEN_SWITCH.MACRO.SET");
  dpConnect("unLHC_CIRCUIT_qpsMacro_Abort_OpenSwitch_CB", 0, "LHC_CIRCUIT_OPEN_SWITCH.MACRO.ABORT");
  
  qpsGenericFunctions_initSOFT_FE("CircuitMacro", "unLHC_CIRCUIT_qpsMacro_debugCB");
  delay(0, 10);
  unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", "unLHC_CIRCUIT_qpsMacro "+unGenericDpFunctions_getComponentsVersion(makeDynString("unLHC_Circuit"))+" loaded", exceptionInfo);
}

//--------------------------------------------------------------------------------------------------------------------------------
// unLHC_CIRCUIT_qpsMacro_debugCB
/**
Purpose:
unLHC_CIRCUIT_qpsMacro_debugCB callback function: debug function with the SOFT_FE front-end. 
This function returns the following information:
- the list of connected systems g_m_bConnected[dsSystemName[i]]
- the list of executing macro g_mMacroResult
- the list of circuit executing a macro g_dsCircuitRunningMacro
- the list of system involved or still involved in a macro from g_mMacroSystemList

Parameters:
  @param sDpe: string, input, the device DPE
  @param f: int, input, trigger value, 1=show debug

Usage: Public

Constraints:
  . PVSS version: 3.6 
  . PVSS manager type: PVSS00CTRL
  . operating system: WXP and Linux
*/
unLHC_CIRCUIT_qpsMacro_debugCB(string sDp, int f)
{
  int i, len;
  dyn_string ds, dsDev;
  string sKey;
  bool b;
  
  if(f == 1)
  {
    synchronized(g_mMacroResult)
    {
      len = mappinglen(g_mMacroResult);
      dynAppend(ds, getCurrentTime());
      dynAppend(ds, "***** start keys *****");
      for(i=1;i<=len;i++){
        sKey = mappingGetKey(g_mMacroResult, i);
        dsDev = g_mMacroSystemList[sKey];
        dynAppend(ds, "-- Key: " + sKey);
        dynAppend(ds, dsDev);
        dynAppend(ds, "-- end Key: " + sKey);
      }
      dynAppend(ds, "***** end keys *****");
      dynAppend(ds, "***** start circuit *****");
      dsDev=g_dsCircuitRunningMacro;
      dynAppend(ds, dsDev);
      dynAppend(ds, "***** end circuit *****");
      dynAppend(ds, "***** start systems *****");
      len = mappinglen(g_m_bConnected);
      for(i=1;i<=len;i++){
        sKey = mappingGetKey(g_m_bConnected, i);
        b = mappingGetValue(g_m_bConnected, i);
        dynAppend(ds, sKey+" "+b);
      }
      dynAppend(ds, "***** end systems *****");
      dynAppend(ds, "***** start HDS_INTERLOCK & COMM, "+g_iThreadId+" *****");
      dynAppend(ds, "-- last update: " + g_tTimeCounter);
      len = mappinglen(g_mHDS_COMM);
      for(i=1;i<=len;i++){
        dsDev = mappingGetValue(g_mHDS_COMM, i);
        dynAppend(ds, "-- system: " + mappingGetKey(g_mHDS_COMM, i));
        dynAppend(ds, "----- HDS status: " + dsDev[QPS_HDS_INTERLOCK_STATUS]);
        dynAppend(ds, "----- HDS warning: " + dsDev[QPS_HDS_INTERLOCK_WARNING_STATUS]);
        dynAppend(ds, "----- QPS_COMM: " + dsDev[QPS_HDS_COMM_STATUS]);
      }
      dynAppend(ds, "***** end HDS_INTERLOCK & COMM *****");
    }
    sKey = sDp;
    strreplace(sKey, "trigger", "userData");
    sKey = dpSubStr(sKey, DPSUB_SYS_DP_EL);
    dpSet(sKey, ds);
  }
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_registerToAllSystem
/**
Purpose:
Register to all systems

Parameters:

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
  . PVSS version: 3.6 
  . operating system: XP and Linux
  . distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_registerToAllSystem()
{
  dyn_string dsHostName, dsSystemName;
  dyn_int diSystemId, diPortNumber; 
  int i,len, iResult;
  bool bRes, bConnected;
  dyn_string exceptionInfo;
  
  unDistributedControl_getAllDeviceConfig(dsSystemName, diSystemId, dsHostName, diPortNumber);
//DebugN(dsSystemName);
  len = dynlen(dsSystemName);
  for(i=1;i<=len;i++) {
    g_m_bConnected[dsSystemName[i]] = false;
// register this animation to the DistributedComponent, the call back function will be called if the remote system is disconnected
// refer to the unDistributedControl.ctl file
// in a non-distributed system, the register will return bRes=true and nothing in exceptionInfo. 
// but the callBack function will not be called
    unDistributedControl_register("unLHC_CIRCUIT_qpsMacro_systemCB", bRes, bConnected, dsSystemName[i]+":", exceptionInfo);
//DebugN("unLHC_CIRCUIT_qpsMacro_registerToAllSystem", bRes, bConnected, dsSystemName[i], exceptionInfo);
  }
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_systemCB
/**
Purpose:
Callback function trigger whenever the state of a system changes: connection/disconnection
whenever a connection callback is triggered (unLHC_CIRCUIT_qpsMacro_systemCB), each callback 
does a dpConnect to the snapshot macro result (unLHC_CIRCUIT_qpsMacro_ResultCB) 
and zero calibrate QPS macro result (unLHC_CIRCUIT_qpsMacro_ResultCB) if both dp exist.

whenever a disconnection is triggered (unLHC_CIRCUIT_qpsMacro_systemCB), each callback checks if the 
system in disconnection state is in the list of system in use for all the keys of g_mMacroSystemList, 
if so for the corresponding key it sets in g_mMacroResult "Error system disconnected" 
and remove itself from g_mMacroSystemList

Parameters:

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
  . PVSS version: 3.6 
  . operating system: XP and Linux
  . distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_systemCB(string sDp, bool bConnected)
{
  dyn_string exceptionInfo;
  int iRes;
  int pos, iAction;
  string deviceName, deviceNameSystem, deviceName1, deviceName2, deviceName3;
  string sHDS_INTERLOCK_DP, sQPS_COMM_DP;
  string sDevA, sDevB, sDevP, sDevT;

  deviceName = dpSubStr(sDp, DPSUB_SYS_DP);
  // get the systemName of the register DP
  deviceNameSystem = substr(deviceName, strlen(getSystemName()+"_unDistributedControl_", strlen(deviceName)));
//DebugN("unLHC_CIRCUIT_qpsMacro_systemCB", deviceNameSystem, g_m_bConnected);
  if(!mappingHasKey(g_m_bConnected, deviceNameSystem)) // the system is not in used return --> should never happen
    return;

  deviceName = deviceNameSystem+QPS_SNAPSHOT_MACRO_RESULT_DPE;
  unGenericObject_Connection(deviceName, g_m_bConnected[deviceNameSystem], bConnected, iAction, exceptionInfo);
//DebugN("unLHC_CIRCUIT_qpsMacro_systemCB", deviceName, iAction);
  switch (iAction)
  {
    case UN_ACTION_DISCONNECT:
      synchronized(g_mHDS_COMM)
      {
        g_mHDS_COMM[deviceNameSystem+":"] = makeDynString("", "", "");
      }
      break;
    case UN_ACTION_DPCONNECT: // deviceName exists and the system is connected
      deviceName1=deviceNameSystem+QPS_ZERO_CALIBRATE_ON_MACRO_RESULT_DPE;
      deviceName2=deviceNameSystem+QPS_RESET_MACRO_RESULT_DPE;
	     deviceName3 = deviceNameSystem+QPS_CSCM_MACRO_RESULT_DPE;
      sDevA=deviceNameSystem+QPS_SELECT_BOARD_A_MACRO_RESULT_DPE;
      sDevB=deviceNameSystem+QPS_SELECT_BOARD_B_MACRO_RESULT_DPE;
      sDevP=deviceNameSystem+QPS_PREPARE_TEST_MODE_MACRO_RESULT_DPE;
      sDevT=deviceNameSystem+QPS_CANCEL_TEST_MODE_MACRO_RESULT_DPE;
      sHDS_INTERLOCK_DP = deviceNameSystem+":"+deviceNameSystem+"_"+QPS_HDS_INTERLOCK_NAME;
      sQPS_COMM_DP = deviceNameSystem+":"+deviceNameSystem+"_"+QPS_COMM_FAULT_NAME;
      if(dpExists(deviceName1) && dpExists(deviceName2) && dpExists(deviceName3)
        && dpExists(sDevA) && dpExists(sDevB) && dpExists(sDevP) && dpExists(sDevT)) { // deviceName1 exists --> do dpConnect to the result DPs
        dpConnect("unLHC_CIRCUIT_qpsMacro_ResultCB", deviceName);
        dpConnect("unLHC_CIRCUIT_qpsMacro_ResultCB", deviceName1);
        dpConnect("unLHC_CIRCUIT_qpsMacro_ResultCB", deviceName2);
        dpConnect("unLHC_CIRCUIT_qpsMacro_ResultCB", deviceName3);
        dpConnect("unLHC_CIRCUIT_qpsMacro_ResultCB", sDevA);
        dpConnect("unLHC_CIRCUIT_qpsMacro_ResultCB", sDevB);
        dpConnect("unLHC_CIRCUIT_qpsMacro_ResultCB", sDevP);
        dpConnect("unLHC_CIRCUIT_qpsMacro_ResultCB", sDevT);
        synchronized(g_mHDS_COMM)
        {
          g_mHDS_COMM[deviceNameSystem+":"] = makeDynString("", "", "");
        }
        dpConnect("unLHC_CIRCUIT_qpsMacro_HDS_COMM_CB", 
                  sHDS_INTERLOCK_DP+".status", sHDS_INTERLOCK_DP+".warningStatus",
                  sQPS_COMM_DP+".status");
        g_m_bConnected[deviceNameSystem] = true;
      }
      break;
    case UN_ACTION_DPDISCONNECT_DISCONNECT:
      deviceName1=deviceNameSystem+QPS_ZERO_CALIBRATE_ON_MACRO_RESULT_DPE;
      deviceName2=deviceNameSystem+QPS_RESET_MACRO_RESULT_DPE;
	     deviceName3 = deviceNameSystem+QPS_CSCM_MACRO_RESULT_DPE;
      sDevA=deviceNameSystem+QPS_SELECT_BOARD_A_MACRO_RESULT_DPE;
      sDevB=deviceNameSystem+QPS_SELECT_BOARD_B_MACRO_RESULT_DPE;
      sDevP=deviceNameSystem+QPS_PREPARE_TEST_MODE_MACRO_RESULT_DPE;
      sDevT=deviceNameSystem+QPS_CANCEL_TEST_MODE_MACRO_RESULT_DPE;
      sHDS_INTERLOCK_DP = deviceNameSystem+":"+deviceNameSystem+"_"+QPS_HDS_INTERLOCK_NAME;
      sQPS_COMM_DP = deviceNameSystem+":"+deviceNameSystem+"_"+QPS_COMM_FAULT_NAME;
      dpDisconnect("unLHC_CIRCUIT_qpsMacro_ResultCB", deviceName);
      dpDisconnect("unLHC_CIRCUIT_qpsMacro_ResultCB", deviceName1);
      dpDisconnect("unLHC_CIRCUIT_qpsMacro_ResultCB", deviceName2);
      dpDisconnect("unLHC_CIRCUIT_qpsMacro_ResultCB", deviceName3);
      dpDisconnect("unLHC_CIRCUIT_qpsMacro_ResultCB", sDevA);
      dpDisconnect("unLHC_CIRCUIT_qpsMacro_ResultCB", sDevB);
      dpDisconnect("unLHC_CIRCUIT_qpsMacro_ResultCB", sDevP);
      dpDisconnect("unLHC_CIRCUIT_qpsMacro_ResultCB", sDevT);
      synchronized(g_mHDS_COMM)
      {
        g_mHDS_COMM[deviceNameSystem+":"] = makeDynString("", "", "");
      }
      dpDisconnect("unLHC_CIRCUIT_qpsMacro_HDS_COMM_CB", 
                  sHDS_INTERLOCK_DP+".status", sHDS_INTERLOCK_DP+".warningStatus",
                  sQPS_COMM_DP+".status");
      
      synchronized(g_mMacroResult)
      {
        unLHC_CIRCUIT_qpsMacro_removeSystem(deviceNameSystem);
      }
      g_m_bConnected[deviceNameSystem] = false;
    break;
  case UN_ACTION_NOTHING:
    default:
    break;
  }
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_HDS_COMM_CB
/**
Purpose:
Callback function triggered when a QPS system has a HDS INTERLOCK or a QPS_COMM problem. 
it sets the result into g_mHDS_COMM for the given system.
the summary state is calulated in separate thread: unLHC_CIRCUIT_qpsMacro_calculateState_HDS_COMM

Parameters:
  sDp1: string, input, CB dp
  dsResult: dyn_string, input, the result

Usage: Public

PVSS manager usage: CTRL 

Constraints:
  . PVSS version: 3.11-SP1 
  . operating system: windows and linux
  . distributed system: yes.
*/
unLHC_CIRCUIT_qpsMacro_HDS_COMM_CB(string sHDS_INTERLOCK_statusDP, string sHDS_INTERLOCK_status,
                                   string sHDS_INTERLOCK_warningStatusDP, string sHDS_INTERLOCK_warningStatus,
                                   string sQPS_COMM_statusDP, string sQPS_COMM_status)
{
  dyn_string dsState;
  string sSystemName = dpSubStr(sHDS_INTERLOCK_statusDP, DPSUB_SYS);
  dsState[QPS_HDS_INTERLOCK_STATUS] = sHDS_INTERLOCK_status;
  dsState[QPS_HDS_INTERLOCK_WARNING_STATUS] = sHDS_INTERLOCK_warningStatus;
  dsState[QPS_HDS_COMM_STATUS] = sQPS_COMM_status;
  synchronized(g_mHDS_COMM)
  {
    g_mHDS_COMM[sSystemName] = dsState;
  }
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_calculateState_HDS_COMM
/**
Purpose:
Calculate the HDS_INTERLOCK status & warning status, and the QPS_COMM status for each sector.

Parameters:
  sDp1: string, input, CB dp
  dsResult: dyn_string, input, the result

Usage: Public

PVSS manager usage: CTRL 

Constraints:
  . PVSS version: 3.11-SP1 
  . operating system: windows and linux
  . distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_calculateState_HDS_COMM()
{
  dyn_string dsOldHDS_INTERLOCK_status, dsOldHDS_INTERLOCK_warningStatus, dsOldQPS_COMM_status;
  string sHDS_INTERLOCK_status, sHDS_INTERLOCK_warningStatus, sQPS_COMM_status;
  int i, len, iSystem, lenSystem;
  mapping mHDS_COMM;
  dyn_string dsData, dsSystem, exceptionInfo;
  mapping m_system;
  
  dsOldHDS_INTERLOCK_status = dsOldHDS_INTERLOCK_warningStatus = dsOldQPS_COMM_status = makeDynString("-","-","-","-","-","-","-","-");
  // build the DPs for each sector if not already existing
  unLHC_CIRCUIT_qpsMacro_init_HDS_COMM_DP("QPS_12_"+QPS_HDS_INTERLOCK_NAME, "QPS_12_"+QPS_COMM_FAULT_NAME, exceptionInfo);

  unLHC_CIRCUIT_qpsMacro_init_HDS_COMM_DP("QPS_23_"+QPS_HDS_INTERLOCK_NAME, "QPS_23_"+QPS_COMM_FAULT_NAME, exceptionInfo);

  unLHC_CIRCUIT_qpsMacro_init_HDS_COMM_DP("QPS_34_"+QPS_HDS_INTERLOCK_NAME, "QPS_34_"+QPS_COMM_FAULT_NAME, exceptionInfo);

  unLHC_CIRCUIT_qpsMacro_init_HDS_COMM_DP("QPS_45_"+QPS_HDS_INTERLOCK_NAME, "QPS_45_"+QPS_COMM_FAULT_NAME, exceptionInfo);

  unLHC_CIRCUIT_qpsMacro_init_HDS_COMM_DP("QPS_56_"+QPS_HDS_INTERLOCK_NAME, "QPS_56_"+QPS_COMM_FAULT_NAME, exceptionInfo);

  unLHC_CIRCUIT_qpsMacro_init_HDS_COMM_DP("QPS_67_"+QPS_HDS_INTERLOCK_NAME, "QPS_67_"+QPS_COMM_FAULT_NAME, exceptionInfo);

  unLHC_CIRCUIT_qpsMacro_init_HDS_COMM_DP("QPS_78_"+QPS_HDS_INTERLOCK_NAME, "QPS_78_"+QPS_COMM_FAULT_NAME, exceptionInfo);

  unLHC_CIRCUIT_qpsMacro_init_HDS_COMM_DP("QPS_81_"+QPS_HDS_INTERLOCK_NAME, "QPS_81_"+QPS_COMM_FAULT_NAME, exceptionInfo);

  // assign the QPS system for each sector
  m_system["QPS_12_"] = makeDynString("QPS_12_DT1A:", "QPS_12_DR1BC:", "QPS_12_DR1HI:", "QPS_12_DL2DE:", "QPS_12_DL2JK:", "QPS_12_DT2FL:");
  m_system["QPS_23_"] = makeDynString("QPS_23_DT2AG:", "QPS_23_DR2BC:", "QPS_23_DR2HI:", "QPS_23_DL3DE:", "QPS_23_DL3JK:", "QPS_23_DR3FL:");
  m_system["QPS_34_"] = makeDynString("QPS_34_DR3FL:", "QPS_34_DR3BC:", "QPS_34_DR3HI:", "QPS_34_DL4DE:", "QPS_34_DL4JK:", "QPS_34_DT4FL:");
  m_system["QPS_45_"] = makeDynString("QPS_45_DT4AG:", "QPS_45_DR4BC:", "QPS_45_DR4HI:", "QPS_45_DL5DE:", "QPS_45_DL5JK:", "QPS_45_DT5F:");
  m_system["QPS_56_"] = makeDynString("QPS_56_DT5A:", "QPS_56_DR5BC:", "QPS_56_DR5HI:", "QPS_56_DL6DE:", "QPS_56_DL6JK:", "QPS_56_DT6FL:");
  m_system["QPS_67_"] = makeDynString("QPS_67_DT6AG:", "QPS_67_DR6BC:", "QPS_67_DR6HI:", "QPS_67_DL7DE:", "QPS_67_DL7JK:", "QPS_67_DT7F:");
  m_system["QPS_78_"] = makeDynString("QPS_78_DT7A:", "QPS_78_DR7BC:", "QPS_78_DR7HI:", "QPS_78_DL8DE:", "QPS_78_DL8JK:", "QPS_78_DT8FL:");
  m_system["QPS_81_"] = makeDynString("QPS_81_DT8AG:", "QPS_81_DR8BC:", "QPS_81_DR8HI:", "QPS_81_DL1DE:", "QPS_81_DL1JK:", "QPS_81_DT1F:");
  
  // loop for ever on g_mHDS_COMM
  while(1)
  {
    delay(QPS_HDS_INTERLOCK_WAIT);
    g_tTimeCounter = getCurrentTime();
//    DebugTN("start", mappinglen(g_mHDS_COMM));
    synchronized(g_mHDS_COMM)
    {
      mHDS_COMM = g_mHDS_COMM;
    }
    len = mappinglen(m_system);
//    DebugTN(mappinglen(mHDS_COMM), len);
    for(i=1;i<=len;i++)
    {
      dsSystem = mappingGetValue(m_system, i);
      lenSystem = dynlen(dsSystem);
      sHDS_INTERLOCK_status = "";
      sHDS_INTERLOCK_warningStatus = "";
      sQPS_COMM_status = "";
      dynClear(exceptionInfo);
      for(iSystem = 1;iSystem<=lenSystem;iSystem++)
      {
        dsData = mHDS_COMM[dsSystem[iSystem]];
        if(dsData[QPS_HDS_INTERLOCK_STATUS] !="")
          sHDS_INTERLOCK_status = unLHC_CIRCUIT_qpsMacro_calculateState_HDS_COMM_add_circuit(sHDS_INTERLOCK_status, dsData[QPS_HDS_INTERLOCK_STATUS]);
        if(dsData[QPS_HDS_INTERLOCK_WARNING_STATUS] !="")
          sHDS_INTERLOCK_warningStatus = unLHC_CIRCUIT_qpsMacro_calculateState_HDS_COMM_add_circuit(sHDS_INTERLOCK_warningStatus, dsData[QPS_HDS_INTERLOCK_WARNING_STATUS]);
        if(dsData[QPS_HDS_COMM_STATUS] !="")
          sQPS_COMM_status = unLHC_CIRCUIT_qpsMacro_calculateState_HDS_COMM_add_circuit(sQPS_COMM_status, dsData[QPS_HDS_COMM_STATUS]);
      }
      // remove from sHDS_INTERLOCK_warningStatus circuit that are in sHDS_INTERLOCK_status
      sHDS_INTERLOCK_warningStatus = unLHC_CIRCUIT_qpsMacro_HDS_COMM_remove(sHDS_INTERLOCK_status, sHDS_INTERLOCK_warningStatus);
      if(sHDS_INTERLOCK_status != dsOldHDS_INTERLOCK_status[i])
      {
        dpSet(mappingGetKey(m_system, i)+QPS_HDS_INTERLOCK_NAME+".status", sHDS_INTERLOCK_status);
        unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                           "unLHC_CIRCUIT_qpsMacro_calculateState HDS_INTERLOCK status "+ sHDS_INTERLOCK_status, exceptionInfo);
      }
      if(sHDS_INTERLOCK_warningStatus != dsOldHDS_INTERLOCK_warningStatus[i])
      {
        dpSet(mappingGetKey(m_system, i)+QPS_HDS_INTERLOCK_NAME+".warningStatus", sHDS_INTERLOCK_warningStatus);
        unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                           "unLHC_CIRCUIT_qpsMacro_calculateState HDS_INTERLOCK warningStatus "+ sHDS_INTERLOCK_warningStatus, exceptionInfo);
      }
      if(sQPS_COMM_status != dsOldQPS_COMM_status[i])
      {
        dpSet(mappingGetKey(m_system, i)+QPS_COMM_FAULT_NAME+".status", sQPS_COMM_status);
        unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                           "unLHC_CIRCUIT_qpsMacro_calculateState QPS_COMM status "+ sQPS_COMM_status, exceptionInfo);
      }
      dsOldHDS_INTERLOCK_status[i] = sHDS_INTERLOCK_status;
      dsOldHDS_INTERLOCK_warningStatus[i] = sHDS_INTERLOCK_warningStatus;
      dsOldQPS_COMM_status[i] = sQPS_COMM_status;
//    DebugTN("end syst", i, dsOldHDS_INTERLOCK_status[i], sHDS_INTERLOCK_status);
    }
//    DebugTN("end", dsOldHDS_INTERLOCK_status);
  }
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_init_HDS_COMM_DP
/**
Purpose:
create the internal HDS_INTERLOCK and QPS_COMM DP and set the archive config.

Parameters:
  sQPS_INTERLOCK_Dp: string, input, the HDS interlock DP
  sQPS_COMM_Dp: string, input, QPS_COMM DP
  exceptionInfo: string, output, exception, error

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
  . PVSS version: 3.6 
  . operating system: XP and Linux
  . distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_init_HDS_COMM_DP(string sQPS_INTERLOCK_Dp, string sQPS_COMM_Dp, dyn_string &exceptionInfo)
{
  bool configExists, isActive;
  string archiveClass;
  int archiveType, smoothProcedure;
  float deadband, timeInterval;
  
  unConfigGenericFunctions_createDp(sQPS_INTERLOCK_Dp, QPS_HDS_INTERLOCK_TYPE, exceptionInfo);
  unConfigGenericFunctions_createDp(sQPS_COMM_Dp, QPS_HDS_INTERLOCK_TYPE, exceptionInfo);
  
  if(dpExists("_unMessageText.message")) 
  {
    fwArchive_get("_unMessageText.message", configExists, archiveClass, archiveType, smoothProcedure, deadband, timeInterval, isActive, exceptionInfo);
    if(configExists) {
      fwArchive_set(sQPS_INTERLOCK_Dp+".status", archiveClass, DPATTR_ARCH_PROC_SIMPLESM, DPATTR_COMPARE_OLD_NEW, 0.0, 0, exceptionInfo);
      fwArchive_set(sQPS_INTERLOCK_Dp+".warningStatus", archiveClass, DPATTR_ARCH_PROC_SIMPLESM, DPATTR_COMPARE_OLD_NEW, 0.0, 0, exceptionInfo);
      fwArchive_set(sQPS_COMM_Dp+".status", archiveClass, DPATTR_ARCH_PROC_SIMPLESM, DPATTR_COMPARE_OLD_NEW, 0.0, 0, exceptionInfo);
    }
    else
      fwException_raise(exceptionInfo, "ERROR", QPS_HDS_INTERLOCK_NAME + " cannot not created archive config", "");
  }
  else
    fwException_raise(exceptionInfo, "ERROR", QPS_HDS_INTERLOCK_NAME + " cannot not created archive config", "");
}
//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_HDS_COMM_remove
/**
Purpose:
remove from sHDS_INTERLOCK_warningStatus circuit that are in sHDS_INTERLOCK_status.

Parameters:
  sHDS_INTERLOCK_status: string, input, the circuit list with HDS problem
  sHDS_INTERLOCK_warningStatus: string, input, the circuit list with HDS warning problem
  return value: the sHDS_INTERLOCK_warningStatus without sHDS_INTERLOCK_status

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
  . PVSS version: 3.6 
  . operating system: XP and Linux
  . distributed system: no.
*/
string unLHC_CIRCUIT_qpsMacro_HDS_COMM_remove(string sHDS_INTERLOCK_status, string sHDS_INTERLOCK_warningStatus)
{
  dyn_string dsStatus, dsWarning;
  int i, len, iPos;
  string sResult;
  
  dsStatus = strsplit(sHDS_INTERLOCK_status, ",");
  dsWarning = strsplit(sHDS_INTERLOCK_warningStatus, ",");
  len = dynlen(dsStatus);
  for(i=1;i<=len;i++)
  {
    iPos = dynContains(dsWarning, dsStatus[i]);
    if(iPos >0)
      dynRemove(dsWarning, iPos);
  }
  // rebuild sHDS_INTERLOCK_warningStatus
  len = dynlen(dsWarning);
  for(i=1;i<=len;i++)
  {
    if(sResult == "")
      sResult = dsWarning[i];
    else
      sResult += ","+dsWarning[i];
  }
  return sResult;
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_calculateState_HDS_COMM_add_circuit
/**
Purpose:
add the data into the lsit if it is not already in.

Parameters:
  sCurrentData: string, input, current data
  sToAdd: string, input, the data to add
  return value: the result data

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
  . PVSS version: 3.6 
  . operating system: XP and Linux
  . distributed system: no.
*/
string unLHC_CIRCUIT_qpsMacro_calculateState_HDS_COMM_add_circuit(string sCurrentData, string sToAdd)
{
  string sResult;
  dyn_string dsData = strsplit(sCurrentData, ",");
  int i, len;
  
  dynAppend(dsData, strsplit(sToAdd, ","));
  dynUnique(dsData);
  len = dynlen(dsData);
  for(i=1;i<=len;i++)
  {
    if(sResult == "")
      sResult = dsData[i];
    else
      sResult += "," + dsData[i];
  }
  return sResult;
}
//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_ResultCB
/**
Purpose:
Callback function triggered when a QPS system returns a result of the execution of a macro
When the result of the QPS dp is set, the callback checks if the system is in the list of the system 
in use: g_mMacroSystemList for the key of the result (key = ID+macro): 
it sets the result into g_mMacroResult and remove itself from g_mMacroSystemList 
for the corresponding key.

Parameters:
  sDp1: string, input, CB dp
  dsResult: dyn_string, input, the result

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
  . PVSS version: 3.6 
  . operating system: XP and Linux
  . distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_ResultCB(string sDp1, dyn_string dsResult)
{
  string sKey, sResult;
  dyn_string dsSystem;
  string sSystem = dpSubStr(sDp1, DPSUB_SYS);
  int iPos;
  
  sSystem = substr(sSystem,0, strpos(sSystem, ":"));
//DebugTN("******* unLHC_CIRCUIT_qpsMacro_ResultCB", sDp1, sSystem, dsResult, g_mMacroResult);
  if(dynlen(dsResult) >=2) {
    sKey = dsResult[1];
    sResult = dsResult[2];
    synchronized(g_mMacroResult)
    {
      if(mappingHasKey(g_mMacroResult, sKey)) {
        dsSystem = g_mMacroSystemList[sKey];
        iPos = dynContains(dsSystem, sSystem);
        if(iPos > 0) {
          dynRemove(dsSystem, iPos);
          g_mMacroSystemList[sKey] = dsSystem;
          dsSystem = g_mMacroResult[sKey];
          dynAppend(dsSystem, sResult);
          g_mMacroResult[sKey] = dsSystem;
        }
      }
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_removeSystem
/**
Purpose:
checks if the sSystemName is in the list of system in use for all the keys of g_mMacroSystemList, 
if so for the corresponding key it adds in g_mMacroResult "Error system disconnected" 
and remove itself from g_mMacroSystemList.

Parameters:
  sSystemName: string, input, the system name

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
  . PVSS version: 3.6 
  . operating system: XP and Linux
  . distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_removeSystem(string sSystemName)
{
  int i, len=mappinglen(g_mMacroSystemList), iPos;
  string sKey;
  dyn_string dsSystem;
  
  for(i=1;i<=len;i++) {
    sKey = mappingGetKey(g_mMacroSystemList, i);
    dsSystem = mappingGetValue(g_mMacroSystemList, i);
    iPos = dynContains(dsSystem, sSystemName);
    if(iPos > 0) { //
      dynRemove(dsSystem, iPos);
      g_mMacroSystemList[sKey] = dsSystem;
      dsSystem = g_mMacroResult[sKey];
      dynAppend(dsSystem, "Error system disconnected");
      g_mMacroResult[sKey] = dsSystem;
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_closeSwitchDelayCB
/**
Purpose:
Callback function to set the delay

Parameters:
  sDp1: string, input, CB dp
  iCloseDelay: int, input, close switch delay
  sDp2: string, input, CB dp
  iResetSystDelay: int, input, reset syst delay

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.6 
	. operating system: XP and Linux
	. distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_closeSwitchDelayCB(string sDp1, int iCloseDelay, string sDp2, int iResetSystDelay)
{
  dyn_string exceptionInfo, dsQpsDp;
  int i, len;

//DebugTN("qpsLHC_CircuitMacro_closeSwitchDelayCB", iCloseDelay, iResetSystDelay);
  unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                    "unLHC_CIRCUIT_qpsMacro_closeSwitchDelayCB new delay: close switch "+ iCloseDelay +" new reset_syst "+iResetSystDelay, exceptionInfo);
  // get the list of QPS system
  dsQpsDp = unLHC_CIRCUIT_qpsMacro_getQpsDp();
  len = dynlen(dsQpsDp);
  if(len > 0) {
    // send the list of circuit with g_bResetSyst  
    for(i=1;i<=len;i++) {
      unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                        "unLHC_CIRCUIT_qpsMacro_closeSwitchDelayCB requested sent to:"+dsQpsDp[i], exceptionInfo);
      if(dpExists(dsQpsDp[i]+":QPS_LHC_CIRCUIT.CLOSE_SWITCH.CLOSE_DELAY")) {
        dpSet(dsQpsDp[i]+":QPS_LHC_CIRCUIT.CLOSE_SWITCH.CLOSE_DELAY", iCloseDelay,
              dsQpsDp[i]+":QPS_LHC_CIRCUIT.CLOSE_SWITCH.RESET_SYST_DELAY", iResetSystDelay);
      }
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_closeSwitchAbortCB
/**
Purpose:
Callback function to abort the close switch

Parameters:
  sDp1: string, input, CB dp
  bAbort: int, bool, abort close switch: true=abort/false=continue

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.6 
	. operating system: XP and Linux
	. distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_closeSwitchAbortCB(string sDp1, bool bAbort)
{
  dyn_string exceptionInfo, dsQpsDp;
  int i, len;
  
  if(bAbort) {
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                      "unLHC_CIRCUIT_qpsMacro_closeSwitchAbortCB abort close switch requested", exceptionInfo);
    // get the list of QPS system
    dsQpsDp = unLHC_CIRCUIT_qpsMacro_getQpsDp();
    len = dynlen(dsQpsDp);
    if(len > 0) {
      // send the list of circuit with g_bResetSyst  
      for(i=1;i<=len;i++) {
        unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                          "unLHC_CIRCUIT_qpsMacro_closeSwitchAbortCB requested sent to:"+dsQpsDp[i], exceptionInfo);
        if(dpExists(dsQpsDp[i]+":QPS_LHC_CIRCUIT.CLOSE_SWITCH.ABORT")) {
          dpSet(dsQpsDp[i]+":QPS_LHC_CIRCUIT.CLOSE_SWITCH.ABORT", bAbort);
        }
      }
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_ResetSwitchCB
/**
Purpose:
Callback function to get the reset syst state

Parameters:
  sResetDp: string, input, CB dp
  bResetSyst: bool, input, reset syst before closing the switch, true=do reset syst/false=do not do reset syst

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.6 
	. operating system: XP and Linux
	. distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_ResetSwitchCB(string sResetDp, bool bResetSyst) synchronized (g_bResetSyst)
{
  g_bResetSyst = bResetSyst;
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_closeSwitchCB
/**
Purpose:
Callback function to close the switch

Parameters:
  sCircuitDp: string, input, CB dp
  dsCircuitList: dyn_string, input, list of circuit

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.6 
	. operating system: XP and Linux
	. distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_closeSwitchCB(string sCircuitDp, dyn_string dsCircuitList) synchronized (g_bResetSyst)
{
  dyn_string dsCircuit, exceptionInfo, dsQpsDp;
  int i, len;
  
  unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                    "unLHC_CIRCUIT_qpsMacro_closeSwitchCB requested:"+dsCircuitList, exceptionInfo);
  // get the list of circuit
  dsCircuit = unLHC_CIRCUIT_qpsMacro_getCircuit(dsCircuitList);
  if(dynlen(dsCircuit) > 0) {
    // get the list of QPS system
    dsQpsDp = unLHC_CIRCUIT_qpsMacro_getQpsDp();
    len = dynlen(dsQpsDp);
    if(len > 0) {
      // send the list of circuit with g_bResetSyst  
      for(i=1;i<=len;i++) {
        unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                          "unLHC_CIRCUIT_qpsMacro_closeSwitchCB requested sent to:"+dsQpsDp[i], exceptionInfo);
        if(dpExists(dsQpsDp[i]+":QPS_LHC_CIRCUIT.CLOSE_SWITCH.CIRCUIT_LIST")) {
          dpSet(dsQpsDp[i]+":QPS_LHC_CIRCUIT.CLOSE_SWITCH.CIRCUIT_LIST", dsCircuit,
                dsQpsDp[i]+":QPS_LHC_CIRCUIT.CLOSE_SWITCH.RESET_SYST", g_bResetSyst);
        }
      }
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unLHC_CIRCUIT_qpsMacro_Snapshot_CB
/**
Purpose:
Callback function to execute the snapshot macro

Parameters:
  sCircuitDp: string, input, CB dp
  dsCircuitListIn: dyn_string, input, list of circuit

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
  . PVSS version: 3.6 
  . operating system: XP and Linux
  . distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_Snapshot_CB(string sCircuitDp, dyn_string dsCircuitListIn)
{
  dyn_string dsCircuit, exceptionInfo, dsQpsDp, dsCircuitList=dsCircuitListIn;
  int i, len;
  string sID;
  
  if(dynlen(dsCircuitList)>=1) {
    sID=dsCircuitListIn[1];
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                      "unLHC_CIRCUIT_qpsMacro_Snapshot_CB requested:"+dsCircuitList, exceptionInfo);
    // remove the id
    dynRemove(dsCircuitList, 1);
    // get the list of circuit
    dsCircuit = unLHC_CIRCUIT_qpsMacro_getCircuit(dsCircuitList);
    if(dynlen(dsCircuit) > 0) {
      startThread("unLHC_CIRCUIT_qpsMacro_executeMacro", sID, "Snapshot", dsCircuit, 
                                                ":QPS_LHC_CIRCUIT_SNAPSHOT.MACRO.SET",
                                                ":QPS_LHC_CIRCUIT_SNAPSHOT.MACRO.RESULT",
                                                "LHC_CIRCUIT_SNAPSHOT.MACRO.RESULT",
                                                LHC_CIRCUITMACRO_SNAPSHOT_TIMEOUT);
    }
    else { // error --> return error if no circuit
      dpSet("LHC_CIRCUIT_SNAPSHOT.MACRO.RESULT", makeDynString(sID, "Error no circuit"));
    }
  }
  else {
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "ERROR", 
                      "unLHC_CIRCUIT_qpsMacro_Snapshot_CB requested with no circuit", exceptionInfo);
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unLHC_CIRCUIT_qpsMacro_Abort_Snapshot_CB
/**
Purpose:
Callback function to get the abort the Snapshot macro

Parameters:
  sDp: string, input, CB dp
  bAbort: bool, input, abort the Snapshot macro, true=abort the Snapshot macro/false=do not abort the Snapshot macro

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.6 
	. operating system: XP and Linux
	. distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_Abort_Snapshot_CB(string sDp, bool bAbort)
{
  dyn_string exceptionInfo, dsQpsDp;
  int i, len;
  
  if(bAbort) {
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                      "unLHC_CIRCUIT_qpsMacro_Abort_Snapshot_CB requested", exceptionInfo);
    // get the list of QPS system
    dsQpsDp = unLHC_CIRCUIT_qpsMacro_getQpsDp();
    len = dynlen(dsQpsDp);
    if(len > 0) {
      for(i=1;i<=len;i++) {
        unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                          "unLHC_CIRCUIT_qpsMacro_Abort_Snapshot_CB requested sent to:"+dsQpsDp[i], exceptionInfo);
        if(dpExists(dsQpsDp[i]+":QPS_LHC_CIRCUIT_SNAPSHOT.MACRO.ABORT")) {
          dpSet(dsQpsDp[i]+":QPS_LHC_CIRCUIT_SNAPSHOT.MACRO.ABORT", bAbort);
        }
      }
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_Zero_Calibrate_ON_CB
/**
Purpose:
Callback function to execute the snapshot macro

Parameters:
  sCircuitDp: string, input, CB dp
  dsCircuitListIn: dyn_string, input, list of circuit

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
  . PVSS version: 3.6 
  . operating system: XP and Linux
  . distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_Zero_Calibrate_ON_CB(string sCircuitDp, dyn_string dsCircuitListIn)
{
  dyn_string dsCircuit, exceptionInfo, dsQpsDp, dsCircuitList=dsCircuitListIn;
  int i, len;
  string sID;
  
  if(dynlen(dsCircuitList)>=1) {
    sID=dsCircuitListIn[1];
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                      "unLHC_CIRCUIT_qpsMacro_Zero_Calibrate_ON_CB requested:"+dsCircuitList, exceptionInfo);
    // remove the id
    dynRemove(dsCircuitList, 1);
    // get the list of circuit
    dsCircuit = unLHC_CIRCUIT_qpsMacro_getCircuit(dsCircuitList);
    if(dynlen(dsCircuit) > 0) {
      startThread("unLHC_CIRCUIT_qpsMacro_executeMacro", sID, "Zero_Calibrate_ON", dsCircuit, 
                                                ":QPS_LHC_CIRCUIT_ZEROCALIBRATEON.MACRO.SET",
                                                ":QPS_LHC_CIRCUIT_ZEROCALIBRATEON.MACRO.RESULT",
                                                "LHC_CIRCUIT_ZEROCALIBRATEON.MACRO.RESULT",
                                                LHC_CIRCUITMACRO_AERO_CALIBRATE_ON_TIMEOUT);
    }
    else { // error --> return error if no circuit
      dpSet("LHC_CIRCUIT_ZEROCALIBRATEON.MACRO.RESULT", makeDynString(sID, "Error no circuit"));
    }
  }
  else {
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "ERROR", 
                      "unLHC_CIRCUIT_qpsMacro_Zero_Calibrate_ON_CB requested with no circuit", exceptionInfo);
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unLHC_CIRCUIT_qpsMacro_Abort_Zero_Calibrate_ON_CB
/**
Purpose:
Callback function to get the abort the Zero_Calibrate_ON macro

Parameters:
  sDp: string, input, CB dp
  bAbort: bool, input, abort the Zero_Calibrate_ON macro, true=abort the Zero_Calibrate_ON macro/false=do not abort the Zero_Calibrate_ON macro

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.6 
	. operating system: XP and Linux
	. distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_Abort_Zero_Calibrate_ON_CB(string sDp, bool bAbort)
{
  dyn_string exceptionInfo, dsQpsDp;
  int i, len;
  
  if(bAbort) {
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                      "unLHC_CIRCUIT_qpsMacro_Abort_Zero_Calibrate_ON_CB requested", exceptionInfo);
    // get the list of QPS system
    dsQpsDp = unLHC_CIRCUIT_qpsMacro_getQpsDp();
    len = dynlen(dsQpsDp);
    if(len > 0) {
      for(i=1;i<=len;i++) {
        unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                          "unLHC_CIRCUIT_qpsMacro_Abort_Zero_Calibrate_ON_CB requested sent to:"+dsQpsDp[i], exceptionInfo);
        if(dpExists(dsQpsDp[i]+":QPS_LHC_CIRCUIT_ZEROCALIBRATEON.MACRO.ABORT")) {
          dpSet(dsQpsDp[i]+":QPS_LHC_CIRCUIT_ZEROCALIBRATEON.MACRO.ABORT", bAbort);
        }
      }
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_CSCM_CB
/**
Purpose:
Callback function to execute the CSCM macro

Parameters:
  sCircuitDp: string, input, CB dp
  dsCircuitListIn: dyn_string, input, list of circuit

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
  . PVSS version: 3.6 
  . operating system: XP and Linux
  . distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_CSCM_CB(string sCircuitDp, dyn_string dsCircuitListIn)
{
  dyn_string dsCircuit, exceptionInfo, dsQpsDp, dsCircuitList=dsCircuitListIn;
  int i, len;
  string sID;
  
  if(dynlen(dsCircuitList)>=1) {
    sID=dsCircuitListIn[1];
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                      "unLHC_CIRCUIT_qpsMacro_CSCM_CB requested:"+dsCircuitList, exceptionInfo);
    // remove the id
    dynRemove(dsCircuitList, 1);
    // get the list of circuit
    dsCircuit = unLHC_CIRCUIT_qpsMacro_getCircuit(dsCircuitList);
    if(dynlen(dsCircuit) > 0) {
      startThread("unLHC_CIRCUIT_qpsMacro_executeMacro", sID, "CSCM", dsCircuit, 
                                                ":QPS_LHC_CIRCUIT_CSCM.MACRO.SET",
                                                ":QPS_LHC_CIRCUIT_CSCM.MACRO.RESULT",
                                                "LHC_CIRCUIT_CSCM.MACRO.RESULT",
                                                LHC_CIRCUITMACRO_AERO_CALIBRATE_ON_TIMEOUT);
    }
    else { // error --> return error if no circuit
      dpSet("LHC_CIRCUIT_CSCM.MACRO.RESULT", makeDynString(sID, "Error no circuit"));
    }
  }
  else {
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "ERROR", 
                      "unLHC_CIRCUIT_qpsMacro_CSCM_CB requested with no circuit", exceptionInfo);
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unLHC_CIRCUIT_qpsMacro_Abort_CSCM_CB
/**
Purpose:
Callback function to get the abort the CSCM macro

Parameters:
  sDp: string, input, CB dp
  bAbort: bool, input, abort the CSCM macro, true=abort the CSCM macro/false=do not abort the CSCM macro

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.6 
	. operating system: XP and Linux
	. distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_Abort_CSCM_CB(string sDp, bool bAbort)
{
  dyn_string exceptionInfo, dsQpsDp;
  int i, len;
  
  if(bAbort) {
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                      "unLHC_CIRCUIT_qpsMacro_Abort_CSCM_CB requested", exceptionInfo);
    // get the list of QPS system
    dsQpsDp = unLHC_CIRCUIT_qpsMacro_getQpsDp();
    len = dynlen(dsQpsDp);
    if(len > 0) {
      for(i=1;i<=len;i++) {
        unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                          "unLHC_CIRCUIT_qpsMacro_Abort_CSCM_CB requested sent to:"+dsQpsDp[i], exceptionInfo);
        if(dpExists(dsQpsDp[i]+":QPS_LHC_CIRCUIT_CSCM.MACRO.ABORT")) {
          dpSet(dsQpsDp[i]+":QPS_LHC_CIRCUIT_CSCM.MACRO.ABORT", bAbort);
        }
      }
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_Reset_CB
/**
Purpose:
Callback function to execute the reset macro

Parameters:
  sCircuitDp: string, input, CB dp
  dsCircuitListIn: dyn_string, input, list of circuit

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
  . PVSS version: 3.6 
  . operating system: XP and Linux
  . distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_Reset_CB(string sCircuitDp, dyn_string dsCircuitListIn)
{
  dyn_string dsCircuit, exceptionInfo, dsQpsDp, dsCircuitList=dsCircuitListIn;
  int i, len;
  string sID;
  
  if(dynlen(dsCircuitList)>=1) {
    sID=dsCircuitListIn[1];
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                      "unLHC_CIRCUIT_qpsMacro_Reset_CB requested:"+dsCircuitList, exceptionInfo);
    // remove the id
    dynRemove(dsCircuitList, 1);
    // get the list of circuit
    dsCircuit = unLHC_CIRCUIT_qpsMacro_getCircuit(dsCircuitList);
    if(dynlen(dsCircuit) > 0) {
      startThread("unLHC_CIRCUIT_qpsMacro_executeMacro", sID, "Reset", dsCircuit, 
                                                ":QPS_LHC_CIRCUIT_RESET.MACRO.SET",
                                                ":QPS_LHC_CIRCUIT_RESET.MACRO.RESULT",
                                                "LHC_CIRCUIT_RESET.MACRO.RESULT",
                                                LHC_CIRCUITMACRO_RESET_TIMEOUT);
    }
    else { // error --> return error if no circuit
      dpSet("LHC_CIRCUIT_RESET.MACRO.RESULT", makeDynString(sID, "Error no circuit"));
    }
  }
  else {
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "ERROR", 
                      "unLHC_CIRCUIT_qpsMacro_Reset_CB requested with no circuit", exceptionInfo);
  }
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_SelectBoardA_CB
/**
Purpose:
Callback function to execute the SelectBoardA macro

Parameters:
  sCircuitDp: string, input, CB dp
  dsCircuitListIn: dyn_string, input, list of circuit

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
  . PVSS version: 3.6 
  . operating system: XP and Linux
  . distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_SelectBoardA_CB(string sCircuitDp, dyn_string dsCircuitListIn)
{
  dyn_string dsCircuit, exceptionInfo, dsQpsDp, dsCircuitList=dsCircuitListIn;
  int i, len;
  string sID;
  
  if(dynlen(dsCircuitList)>=1) {
    sID=dsCircuitListIn[1];
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                      "unLHC_CIRCUIT_qpsMacro_SelectBoardA_CB requested:"+dsCircuitList, exceptionInfo);
    // remove the id
    dynRemove(dsCircuitList, 1);
    // get the list of circuit
    dsCircuit = unLHC_CIRCUIT_qpsMacro_getCircuit(dsCircuitList);
    if(dynlen(dsCircuit) > 0) {
      startThread("unLHC_CIRCUIT_qpsMacro_executeMacro", sID, "SelectBoardA", dsCircuit, 
                                                ":QPS_LHC_CIRCUIT_SELECT_BOARD_A.MACRO.SET",
                                                ":QPS_LHC_CIRCUIT_SELECT_BOARD_A.MACRO.RESULT",
                                                "LHC_CIRCUIT_SELECT_BOARD_A.MACRO.RESULT",
                                                LHC_CIRCUITMACRO_SELECT_BOARD_TIMEOUT);
    }
    else { // error --> return error if no circuit
      dpSet("LHC_CIRCUIT_SELECT_BOARD_A.MACRO.RESULT", makeDynString(sID, "Error no circuit"));
    }
  }
  else {
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "ERROR", 
                      "unLHC_CIRCUIT_qpsMacro_SelectBoardA_CB requested with no circuit", exceptionInfo);
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unLHC_CIRCUIT_qpsMacro_Abort_SelectBoardA_CB
/**
Purpose:
Callback function to get the abort the SelectBoardA macro

Parameters:
  sDp: string, input, CB dp
  bAbort: bool, input, abort the SelectBoardA macro, true=abort the SelectBoardA macro/false=do not abort the SelectBoardA macro

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.6 
	. operating system: XP and Linux
	. distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_Abort_SelectBoardA_CB(string sDp, bool bAbort)
{
  dyn_string exceptionInfo, dsQpsDp;
  int i, len;
  
  if(bAbort) {
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                      "unLHC_CIRCUIT_qpsMacro_Abort_SelectBoardA_CB requested", exceptionInfo);
    // get the list of QPS system
    dsQpsDp = unLHC_CIRCUIT_qpsMacro_getQpsDp();
    len = dynlen(dsQpsDp);
    if(len > 0) {
      for(i=1;i<=len;i++) {
        unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                          "unLHC_CIRCUIT_qpsMacro_Abort_SelectBoardA_CB requested sent to:"+dsQpsDp[i], exceptionInfo);
        if(dpExists(dsQpsDp[i]+":QPS_LHC_CIRCUIT_SELECT_BOARD_A.MACRO.ABORT")) {
          dpSet(dsQpsDp[i]+":QPS_LHC_CIRCUIT_SELECT_BOARD_A.MACRO.ABORT", bAbort);
        }
      }
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_SelectBoardB_CB
/**
Purpose:
Callback function to execute the SelectBoardB macro

Parameters:
  sCircuitDp: string, input, CB dp
  dsCircuitListIn: dyn_string, input, list of circuit

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
  . PVSS version: 3.6 
  . operating system: XP and Linux
  . distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_SelectBoardB_CB(string sCircuitDp, dyn_string dsCircuitListIn)
{
  dyn_string dsCircuit, exceptionInfo, dsQpsDp, dsCircuitList=dsCircuitListIn;
  int i, len;
  string sID;
  
  if(dynlen(dsCircuitList)>=1) {
    sID=dsCircuitListIn[1];
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                      "unLHC_CIRCUIT_qpsMacro_SelectBoardB_CB requested:"+dsCircuitList, exceptionInfo);
    // remove the id
    dynRemove(dsCircuitList, 1);
    // get the list of circuit
    dsCircuit = unLHC_CIRCUIT_qpsMacro_getCircuit(dsCircuitList);
    if(dynlen(dsCircuit) > 0) {
      startThread("unLHC_CIRCUIT_qpsMacro_executeMacro", sID, "SelectBoardB", dsCircuit, 
                                                ":QPS_LHC_CIRCUIT_SELECT_BOARD_B.MACRO.SET",
                                                ":QPS_LHC_CIRCUIT_SELECT_BOARD_B.MACRO.RESULT",
                                                "LHC_CIRCUIT_SELECT_BOARD_B.MACRO.RESULT",
                                                LHC_CIRCUITMACRO_SELECT_BOARD_TIMEOUT);
    }
    else { // error --> return error if no circuit
      dpSet("LHC_CIRCUIT_SELECT_BOARD_B.MACRO.RESULT", makeDynString(sID, "Error no circuit"));
    }
  }
  else {
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "ERROR", 
                      "unLHC_CIRCUIT_qpsMacro_SelectBoardB_CB requested with no circuit", exceptionInfo);
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unLHC_CIRCUIT_qpsMacro_Abort_SelectBoardB_CB
/**
Purpose:
Callback function to get the abort the SelectBoardB macro

Parameters:
  sDp: string, input, CB dp
  bAbort: bool, input, abort the SelectBoardB macro, true=abort the SelectBoardB macro/false=do not abort the SelectBoardB macro

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.6 
	. operating system: XP and Linux
	. distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_Abort_SelectBoardB_CB(string sDp, bool bAbort)
{
  dyn_string exceptionInfo, dsQpsDp;
  int i, len;
  
  if(bAbort) {
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                      "unLHC_CIRCUIT_qpsMacro_Abort_SelectBoardB_CB requested", exceptionInfo);
    // get the list of QPS system
    dsQpsDp = unLHC_CIRCUIT_qpsMacro_getQpsDp();
    len = dynlen(dsQpsDp);
    if(len > 0) {
      for(i=1;i<=len;i++) {
        unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                          "unLHC_CIRCUIT_qpsMacro_Abort_SelectBoardB_CB requested sent to:"+dsQpsDp[i], exceptionInfo);
        if(dpExists(dsQpsDp[i]+":QPS_LHC_CIRCUIT_SELECT_BOARD_B.MACRO.ABORT")) {
          dpSet(dsQpsDp[i]+":QPS_LHC_CIRCUIT_SELECT_BOARD_B.MACRO.ABORT", bAbort);
        }
      }
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_PrepareTestMode_CB
/**
Purpose:
Callback function to execute the PrepareTestMode macro

Parameters:
  sCircuitDp: string, input, CB dp
  dsCircuitListIn: dyn_string, input, list of circuit

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
  . PVSS version: 3.6 
  . operating system: XP and Linux
  . distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_PrepareTestMode_CB(string sCircuitDp, dyn_string dsCircuitListIn)
{
  dyn_string dsCircuit, exceptionInfo, dsQpsDp, dsCircuitList=dsCircuitListIn;
  int i, len;
  string sID;
  
  if(dynlen(dsCircuitList)>=1) {
    sID=dsCircuitListIn[1];
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                      "unLHC_CIRCUIT_qpsMacro_PrepareTestMode_CB requested:"+dsCircuitList, exceptionInfo);
    // remove the id
    dynRemove(dsCircuitList, 1);
    // get the list of circuit
    dsCircuit = unLHC_CIRCUIT_qpsMacro_getCircuit(dsCircuitList);
    if(dynlen(dsCircuit) > 0) {
      startThread("unLHC_CIRCUIT_qpsMacro_executeMacro", sID, "PrepareTestMode", dsCircuit, 
                                                ":QPS_LHC_CIRCUIT_PREPARE_TEST_MODE.MACRO.SET",
                                                ":QPS_LHC_CIRCUIT_PREPARE_TEST_MODE.MACRO.RESULT",
                                                "LHC_CIRCUIT_PREPARE_TEST_MODE.MACRO.RESULT",
                                                LHC_CIRCUITMACRO_TEST_MODE_TIMEOUT);
    }
    else { // error --> return error if no circuit
      dpSet("LHC_CIRCUIT_PREPARE_TEST_MODE.MACRO.RESULT", makeDynString(sID, "Error no circuit"));
    }
  }
  else {
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "ERROR", 
                      "unLHC_CIRCUIT_qpsMacro_PrepareTestMode_CB requested with no circuit", exceptionInfo);
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unLHC_CIRCUIT_qpsMacro_Abort_PrepareTestMode_CB
/**
Purpose:
Callback function to get the abort the PrepareTestMode macro

Parameters:
  sDp: string, input, CB dp
  bAbort: bool, input, abort the PrepareTestMode macro, true=abort the PrepareTestMode macro/false=do not abort the PrepareTestMode macro

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.6 
	. operating system: XP and Linux
	. distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_Abort_PrepareTestMode_CB(string sDp, bool bAbort)
{
  dyn_string exceptionInfo, dsQpsDp;
  int i, len;
  
  if(bAbort) {
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                      "unLHC_CIRCUIT_qpsMacro_Abort_PrepareTestMode_CB requested", exceptionInfo);
    // get the list of QPS system
    dsQpsDp = unLHC_CIRCUIT_qpsMacro_getQpsDp();
    len = dynlen(dsQpsDp);
    if(len > 0) {
      for(i=1;i<=len;i++) {
        unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                          "unLHC_CIRCUIT_qpsMacro_Abort_PrepareTestMode_CB requested sent to:"+dsQpsDp[i], exceptionInfo);
        if(dpExists(dsQpsDp[i]+":QPS_LHC_CIRCUIT_PREPARE_TEST_MODE.MACRO.ABORT")) {
          dpSet(dsQpsDp[i]+":QPS_LHC_CIRCUIT_PREPARE_TEST_MODE.MACRO.ABORT", bAbort);
        }
      }
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_CancelTestMode_CB
/**
Purpose:
Callback function to execute the CancelTestMode macro

Parameters:
  sCircuitDp: string, input, CB dp
  dsCircuitListIn: dyn_string, input, list of circuit

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
  . PVSS version: 3.6 
  . operating system: XP and Linux
  . distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_CancelTestMode_CB(string sCircuitDp, dyn_string dsCircuitListIn)
{
  dyn_string dsCircuit, exceptionInfo, dsQpsDp, dsCircuitList=dsCircuitListIn;
  int i, len;
  string sID;
  
  if(dynlen(dsCircuitList)>=1) {
    sID=dsCircuitListIn[1];
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                      "unLHC_CIRCUIT_qpsMacro_CancelTestMode_CB requested:"+dsCircuitList, exceptionInfo);
    // remove the id
    dynRemove(dsCircuitList, 1);
    // get the list of circuit
    dsCircuit = unLHC_CIRCUIT_qpsMacro_getCircuit(dsCircuitList);
    if(dynlen(dsCircuit) > 0) {
      startThread("unLHC_CIRCUIT_qpsMacro_executeMacro", sID, "CancelTestMode", dsCircuit, 
                                                ":QPS_LHC_CIRCUIT_CANCEL_TEST_MODE.MACRO.SET",
                                                ":QPS_LHC_CIRCUIT_CANCEL_TEST_MODE.MACRO.RESULT",
                                                "LHC_CIRCUIT_CANCEL_TEST_MODE.MACRO.RESULT",
                                                LHC_CIRCUITMACRO_TEST_MODE_TIMEOUT);
    }
    else { // error --> return error if no circuit
      dpSet("LHC_CIRCUIT_CANCEL_TEST_MODE.MACRO.RESULT", makeDynString(sID, "Error no circuit"));
    }
  }
  else {
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "ERROR", 
                      "unLHC_CIRCUIT_qpsMacro_CancelTestMode_CB requested with no circuit", exceptionInfo);
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unLHC_CIRCUIT_qpsMacro_Abort_CancelTestMode_CB
/**
Purpose:
Callback function to get the abort the CancelTestMode macro

Parameters:
  sDp: string, input, CB dp
  bAbort: bool, input, abort the CancelTestMode macro, true=abort the CancelTestMode macro/false=do not abort the CancelTestMode macro

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.6 
	. operating system: XP and Linux
	. distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_Abort_CancelTestMode_CB(string sDp, bool bAbort)
{
  dyn_string exceptionInfo, dsQpsDp;
  int i, len;
  
  if(bAbort) {
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                      "unLHC_CIRCUIT_qpsMacro_Abort_CancelTestMode_CB requested", exceptionInfo);
    // get the list of QPS system
    dsQpsDp = unLHC_CIRCUIT_qpsMacro_getQpsDp();
    len = dynlen(dsQpsDp);
    if(len > 0) {
      for(i=1;i<=len;i++) {
        unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                          "unLHC_CIRCUIT_qpsMacro_Abort_CancelTestMode_CB requested sent to:"+dsQpsDp[i], exceptionInfo);
        if(dpExists(dsQpsDp[i]+":QPS_LHC_CIRCUIT_CANCEL_TEST_MODE.MACRO.ABORT")) {
          dpSet(dsQpsDp[i]+":QPS_LHC_CIRCUIT_CANCEL_TEST_MODE.MACRO.ABORT", bAbort);
        }
      }
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_OpenSwitch_CB
/**
Purpose:
Callback function to open the 13kA switch

Parameters:
  sCircuitDp: string, input, CB dp
  dsCircuitList: dyn_string, input, list of circuit

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.6 
	. operating system: XP and Linux
	. distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_OpenSwitch_CB(string sCircuitDp, dyn_string dsCircuitList)
{
  dyn_string dsCircuit, dsAllQpsDp, dsQpsDp, dsQpsDpEven, dsQpsDpOdd;
  int i, len=dynlen(dsCircuitList);
  dyn_string dsRB, dsRQ;
  
  dsAllQpsDp = unLHC_CIRCUIT_qpsMacro_getQpsDp();
  dsQpsDp = dynPatternMatch("QPS_*DT*", dsAllQpsDp);
  dynAppend(dsQpsDp, dynPatternMatch("QPS_*DR3F*", dsAllQpsDp));
//  DebugTN(dsCircuitList);
  // split in RB and other circuit
  for(i=1;i<=len;i++)
  {
    if(patternMatch("RB.A*", dsCircuitList[i]))
      dynAppend(dsRB, dsCircuitList[i]);
    else
      dynAppend(dsRQ, dsCircuitList[i]);
  }
  
  //split EVEN and ODD QPS dp
  len=dynlen(dsQpsDp);
  for(i=1;i<=len;i++)
  {
    if(patternMatch("QPS_*DT2*", dsQpsDp[i]))
      dynAppend(dsQpsDpEven, dsQpsDp[i]);
    else if(patternMatch("QPS_*DT4*", dsQpsDp[i]))
      dynAppend(dsQpsDpEven, dsQpsDp[i]);
    else if(patternMatch("QPS_*DT6*", dsQpsDp[i]))
      dynAppend(dsQpsDpEven, dsQpsDp[i]);
    else if(patternMatch("QPS_*DT8*", dsQpsDp[i]))
      dynAppend(dsQpsDpEven, dsQpsDp[i]);
    else if(patternMatch("QPS_*DT*", dsQpsDp[i]))
      dynAppend(dsQpsDpOdd, dsQpsDp[i]);
    else if(patternMatch("QPS_*DR3F*", dsQpsDp[i]))
      dynAppend(dsQpsDpOdd, dsQpsDp[i]);
  }
  while(dynlen(dsQpsDpOdd) < dynlen(dsQpsDpEven))
    dynAppend(dsQpsDpOdd, "");
  while(dynlen(dsQpsDpEven) < dynlen(dsQpsDpOdd))
    dynAppend(dsQpsDpEven, "");
    
  dynAppend(dsRB, dsRQ);
  len = dynlen(dsQpsDpEven);
  for(i=1;i<=len;i++)
  {
    delay(0,3); // QPS-219: extra delay to generate unique IDs. DataServer is very fast processing the threads and generating commands with same ID.
    startThread("unLHC_CIRCUIT_qpsMacro_OpenSwitch", dsRB, dsQpsDpEven[i], dsQpsDpOdd[i]);
  }
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_OpenSwitch
/**
Purpose:
function to trigger the open switch macro on a QPS system

Parameters:
  dsCircuitList: dyn_string, input, list of circuit
  sQpsDpEven: string, input, QPS EVEN system DP
  sQpsDpOdd: string, input, QPS ODD system DP

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.6 
	. operating system: XP and Linux
	. distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_OpenSwitch(dyn_string dsCircuitList, string sQpsDpEven, string sQpsDpOdd)
{
  dyn_string dsCircuit, exceptionInfo;
  int i, len;
  bool bOk;
  int iWait;
  int iStartEvenTime, iCurrentTime;
  
  unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                    "unLHC_CIRCUIT_qpsMacro_OpenSwitch requested:"+dsCircuitList+", "+sQpsDpEven+", "+sQpsDpOdd, exceptionInfo);
  // get the list of circuit
  dsCircuit = unLHC_CIRCUIT_qpsMacro_getCircuit(dsCircuitList);
  if(dynlen(dsCircuit) > 0) {
    delay(LHC_CIRCUITMACRO_OPEN_SWITCH_DELAY_BEFORE_START);
    // trigger first Even system
    iStartEvenTime = getCurrentTime();
    dynInsertAt(dsCircuit, iStartEvenTime, 1);
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                      "unLHC_CIRCUIT_qpsMacro_OpenSwitch requested: "+dsCircuit+", sent to:"+sQpsDpEven, exceptionInfo);
    if(dpExists(sQpsDpEven+":QPS_LHC_CIRCUIT_OPEN_SWITCH.MACRO.SET")) {
      dpSet(sQpsDpEven+":QPS_LHC_CIRCUIT_OPEN_SWITCH.MACRO.SET", dsCircuit);
      // wait until completion
      bOk = unLHC_CIRCUIT_qpsMacro_OpenSwitch_getResult(dsCircuit, sQpsDpEven+":QPS_LHC_CIRCUIT_OPEN_SWITCH.MACRO.RESULT");
      // wait QPS_LHC_CIRCUITMACRO_OPEN_SWITCH_MACRO_DELAY if ok
    }
    if(bOk)
      delay(QPS_LHC_CIRCUITMACRO_OPEN_SWITCH_MACRO_DELAY);
    else
    {
      iCurrentTime = getCurrentTime();
      iWait = iStartEvenTime + QPS_LHC_CIRCUITMACRO_OPEN_SWITCH_MACRO_TIMEOUT - iCurrentTime;
      if(iWait > 0)  
        delay(iWait);
    }
    // trigger Odd system
    iCurrentTime = getCurrentTime();
    dsCircuit[1]=iCurrentTime;
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                      "unLHC_CIRCUIT_qpsMacro_OpenSwitch requested: "+dsCircuit+", sent to:"+sQpsDpOdd, exceptionInfo);
    if(dpExists(sQpsDpOdd+":QPS_LHC_CIRCUIT_OPEN_SWITCH.MACRO.SET")) {
      dpSet(sQpsDpOdd+":QPS_LHC_CIRCUIT_OPEN_SWITCH.MACRO.SET", dsCircuit);
      // wait until completion
      bOk = unLHC_CIRCUIT_qpsMacro_OpenSwitch_getResult(dsCircuit, sQpsDpOdd+":QPS_LHC_CIRCUIT_OPEN_SWITCH.MACRO.RESULT");
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_OpenSwitch_getResult
/**
Purpose:
function to the result of an open switch macro on a QPS system

Parameters:
  dsCircuitList: dyn_string, input, list of circuit
  sQpsDp: string, input, QPS EVEN result DP

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.6 
	. operating system: XP and Linux
	. distributed system: no.
*/
bool unLHC_CIRCUIT_qpsMacro_OpenSwitch_getResult(dyn_string dsCircuit, string sQpsDp)
{
  bool bResult=false;
  string sResult;
  dyn_string dsResult, exceptionInfo;
  int i, len=QPS_LHC_CIRCUITMACRO_OPEN_SWITCH_MACRO_TIMEOUT*10 +20;
  string sId = dsCircuit[1], sResultId;
  for(i=1;i<=len;i++)
  {
    dpGet(sQpsDp, dsResult);
    sResultId = "";
    sResult = "";
    if(dynlen(dsResult) >=2)
    {
      sResultId = dsResult[1];
      sResult = dsResult[2];
    }
    if(sId == sResultId)
    {
      switch(sResult)
      {
        case "OK":
        case "Error no device for this circuit":
          bResult = true;
          break;
      }
    }
    if(bResult)
      i = len+1;
    else
      delay(0,100);
  }
  unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                    "unLHC_CIRCUIT_qpsMacro_OpenSwitch_getResult requested: "+dsCircuit+", from to:"+sQpsDp+", res: "+dsResult, exceptionInfo);
  return bResult;
}
//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_Abort_OpenSwitch_CB
/**
Purpose:
Callback function to abort the open switch

Parameters:
  sDp1: string, input, CB dp
  bAbort: int, bool, abort open switch: true=abort/false=continue

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.6 
	. operating system: XP and Linux
	. distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_Abort_OpenSwitch_CB(string sDp1, bool bAbort)
{
  dyn_string exceptionInfo, dsQpsDp;
  int i, len;
  
  if(bAbort) {
    unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                      "unLHC_CIRCUIT_qpsMacro_Abort_OpenSwitch_CB abort close switch requested", exceptionInfo);
    // get the list of QPS system
    dsQpsDp = unLHC_CIRCUIT_qpsMacro_getQpsDp();
    len = dynlen(dsQpsDp);
    if(len > 0) {
      // send the list of circuit with g_bResetSyst  
      for(i=1;i<=len;i++) {
        unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                          "unLHC_CIRCUIT_qpsMacro_Abort_OpenSwitch_CB requested sent to:"+dsQpsDp[i], exceptionInfo);
        if(dpExists(dsQpsDp[i]+":QPS_LHC_CIRCUIT_OPEN_SWITCH.MACRO.ABORT")) {
          dpSet(dsQpsDp[i]+":QPS_LHC_CIRCUIT_OPEN_SWITCH.MACRO.ABORT", bAbort);
        }
      }
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_getCircuit
/**
Purpose:
Get the circuit list

Parameters:
  dsCircuitList: dyn_string, input, list of circuit/pattern
  return value: dyn_string, output, list of circuit
Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.6 
	. operating system: XP and Linux
	. distributed system: no.
*/
dyn_string unLHC_CIRCUIT_qpsMacro_getCircuit(dyn_string dsCircuitList)
{
  dyn_string dsCircuit;
  int i, len= dynlen(dsCircuitList);
  dyn_string dsTemp;
  
//DebugTN("start unLHC_CIRCUIT_qpsMacro_getCircuit", dsCircuitList);
  for(i=1;i<=len;i++) {
    dsTemp = dynPatternMatch(dsCircuitList[i], g_dsLHC_CIRCUITCircuit);
    dynAppend(dsCircuit, dsTemp);
  }
  
  dynUnique(dsCircuit);
//DebugTN("end unLHC_CIRCUIT_qpsMacro_getCircuit", dsCircuit);
  return dsCircuit;
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_getQpsDp
/**
Purpose:
Get the circuit list

Parameters:
  return value: dyn_string, output, list of QPS system
Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.6 
	. operating system: XP and Linux
	. distributed system: no.
*/
dyn_string unLHC_CIRCUIT_qpsMacro_getQpsDp()
{
  dyn_string dsQps, dsSystemNames, dsHost, exInfo;
  dyn_int diSystemId, diPort;
  int i, len;
  bool bConnected;
    
  unDistributedControl_getAllDeviceConfig(dsSystemNames, diSystemId, dsHost, diPort);
  len = dynlen(dsSystemNames);
  for(i=1;i<=len;i++)
  {
    if(strpos(dsSystemNames[i], ":") < 0)
      dsSystemNames[i] = dsSystemNames[i]+":"; 
    if(dsSystemNames[i] != g_sSystemName) {
      unDistributedControl_isConnected(bConnected, dsSystemNames[i]);
      if(bConnected)
        dynAppend(dsQps, substr(dsSystemNames[i],0, strpos(dsSystemNames[i], ":")));
    }
  }
//DebugTN("end unLHC_CIRCUIT_qpsMacro_getQpsDp", dsSystemNames, dsQps, g_sSystemName);
  
  return dsQps;
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_resetAgentCB
/**
Purpose:
Callback function to reset the agent

Parameters:
  sCircuitDp: string, input, CB dp
  dsCircuitList: dyn_string, input, list of circuit

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.6 
	. operating system: XP and Linux
	. distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_resetAgentCB(string sCircuitDp, dyn_string dsCircuitList)
{
  dyn_string dsCircuit, exceptionInfo, dsQpsDp;
  int i, len;
  
  unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                    "unLHC_CIRCUIT_qpsMacro_resetAgentCB requested:"+dsCircuitList, exceptionInfo);
  // get the list of circuit
  dsCircuit = unLHC_CIRCUIT_qpsMacro_getCircuit(dsCircuitList);
  if(dynlen(dsCircuit) > 0) {
    // get the list of QPS system
    dsQpsDp = unLHC_CIRCUIT_qpsMacro_getQpsDp();
    len = dynlen(dsQpsDp);
    if(len > 0) {
      // send the list of circuit with g_bResetSyst  
      for(i=1;i<=len;i++) {
        unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                          "unLHC_CIRCUIT_qpsMacro_resetAgentCB requested sent to:"+dsQpsDp[i], exceptionInfo);
        if(dpExists(dsQpsDp[i]+":QPS_LHC_CIRCUIT.RESET.CIRCUIT_LIST")) {
          dpSet(dsQpsDp[i]+":QPS_LHC_CIRCUIT.RESET.CIRCUIT_LIST", dsCircuit);
        }
      }
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------

//unLHC_CIRCUIT_qpsMacro_executeMacro
/**
Purpose:
Execute a macro: 
  check if the macro was already started
  check if some circuit are already in use
  if all OK get the list of QPS system, trigger them, check the result

Parameters:
  sID: string, input, identification of the macro
  sID: string, input, the unique key to keep trace of the macro currently in execution
  dsCircuit: dyn_string, input, list of circuit
  sQPSSetDpe: string, input, the QPS DPE to set
  sQPSResDpe: string, input, the QPS DPE for the result
  sResDpe: string, input, the DPE for the result
  iTimeout: int, input, max iteration time allowed for the macro

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
  . PVSS version: 3.6 
  . operating system: XP and Linux
  . distributed system: no.
*/
unLHC_CIRCUIT_qpsMacro_executeMacro(string sID, string sMacro, dyn_string dsCircuit, 
                                    string sQPSSetDpe, string sQPSResDpe, string sResDpe,
                                    int iTimeout)
{
  string sResult;
  dyn_string dsCircuitMacro;
  bool bConti=false;
  int i, len;
  dyn_string dsResult, dsResultMacro, dsQpsDp, exceptionInfo, dsQps;
  string sKey = sID+sMacro;
  int iRes, lenRes, nbOfQPS=0;

// if already key in g_mMacroResult --> error
  synchronized(g_mMacroResult)
  {
    if(mappingHasKey(g_mMacroResult, sKey))
    {
      sResult = sID+",current macro already not terminated";
    }
  }
  if(sResult =="") { // no error so far --> continue
  // get all the running QPS and trigger them 
    dsQpsDp = unLHC_CIRCUIT_qpsMacro_getQpsDp();
    len = dynlen(dsQpsDp);
    //if one of the circuit is in g_dsCircuitRunningMacro --> error
//DebugN(sKey, dsQpsDp, dsCircuit, g_dsCircuitRunningMacro);
    if(dynlen(dynIntersect(dsCircuit, g_dsCircuitRunningMacro)) >0)
    {
      sResult = sID+",a macro is not ended for one (or many) QPS system(s)";
    }
    else {
      synchronized(g_mMacroResult)
      {
        dsResult = dsCircuit;
        dynAppend(g_dsCircuitRunningMacro, dsResult);
        dsCircuitMacro = dsCircuit;
        g_mMacroResult[sKey]=makeDynString();

        if(len > 0) {
          bConti = true;
          dynInsertAt(dsCircuitMacro, sKey, 1);
          // send the list of circuit  
          for(i=1;i<=len;i++) {
            unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro", "user", "*", "INFO", 
                              "unLHC_CIRCUIT_qpsMacro_executeMacro "+sMacro+" requested sent to:"+dsQpsDp[i], exceptionInfo);
            if(dpExists(dsQpsDp[i]+sQPSSetDpe) && (strpos(dsQpsDp[i], "QPS") >= 0)) {
              dpSet(dsQpsDp[i]+sQPSSetDpe, dsCircuitMacro);
              nbOfQPS++;
              dynAppend(dsQps, dsQpsDp[i]);
            }
          }
        }
        g_mMacroSystemList[sKey] = dsQps;
        len = dynlen(dsQps);
      }
    // check result
    // if all result of the corresponding sID = "Error no device for this circuit" --> error
    // else if result of the corresponding sID = "Error no device for this circuit" --> consider OK
    // check any other result, if in the check a system is disconnected -> error
    // return back the error
//DebugN("entring in infinite loop ", sKey, bConti, sResult);
      for(i=1;(i<=iTimeout) & bConti;i++) {
        qpsLHC_CircuitMacro_delay();
        synchronized(g_mMacroResult)
        {
          dsResultMacro = g_mMacroResult[sKey];
//DebugTN(i, sKey, dsResultMacro, len);
          if(dynlen(dsResultMacro) == len) { // the macro is done
            dynUnique(dsResultMacro);
            if(dynlen(dsResultMacro) != 1) { // not the same result
              // replace "Error no device for this circuit" by "OK" and check again
              lenRes = dynlen(dsResultMacro);
              for(iRes=1;iRes<=lenRes;iRes++)
              {
                if(dsResultMacro[iRes] == "Error no device for this circuit")
                  dsResultMacro[iRes] = "OK";
              }
              dynUnique(dsResultMacro);
              if(dynlen(dsResultMacro) != 1) // bad, at least the macro was not executed successfully on one QPS system
                dsResultMacro[1] = "Macro not executed successfully on one (many) QPS system(s)";
            }
            bConti = false; // stop the infinite loop
            sResult=sID+","+dsResultMacro[1];
          }
          else { // keep the value
            g_mMacroResult[sKey] = dsResultMacro;
          }
        }
      }
      if(i>=iTimeout)
        sResult=sID+",Error timeout";
    }
    synchronized(g_mMacroResult)
    {
      // remove the key from g_mMacroResult and g_mMacroSystemList
      mappingRemove(g_mMacroResult, sKey);
      mappingRemove(g_mMacroSystemList, sKey);
      // remove the dsCircuit from g_dsCircuitRunningMacro
      len=dynlen(dsCircuit);
      for(i=1;i<=len;i++) {
        iRes = dynContains(g_dsCircuitRunningMacro, dsCircuit[i]);
        if(iRes>0)
          dynRemove(g_dsCircuitRunningMacro, iRes);
      }
    }
  }

  if(sResult == "") // strange should never happen
    sResult = sID+",internal error or no QPS available for the circuit";
  dpSet(sResDpe, strsplit(sResult,","));
  unMessageText_send("*", "*", g_sSystemName+"unLHC_CIRCUIT_qpsMacro_executeMacro", "user", "*", "INFO",
                     "end unLHC_CIRCUIT_qpsMacro_executeMacro "+sMacro+", "+sID+", res: "+sResult, exceptionInfo);
}
//------------------------------------------------------------------------------------------------------------------------

//@}
